#include <climits>
#include <iostream>
#include <map>
#include <tuple>
#include <utility>

struct canvas {
	std::map<std::tuple<int, int>, char> pixels;

	// This might not be the cleanest solution, but it serves nicely to
	// demonstrate the difference between const and non-const methods.

	char& pixel(int x, int y) {
		auto [it, ins] = pixels.try_emplace({ x, y }, ' ');
		return it->second;
	}

	char pixel(int x, int y) const {
		auto it = pixels.find({ x, y });
		return it == pixels.end() ? ' ' : it->second;
	}

	void clear() { pixels.clear(); }

	void draw() const;

	auto bounds() const;
};

auto canvas::bounds() const {
	int min_x = INT_MAX, min_y = INT_MAX,
	    max_x = INT_MIN, max_y = INT_MIN;

	for (auto& [xy, p] : pixels) {
		if (p == ' ') continue;
		auto [x, y] = xy;
		min_x = std::min(min_x, x);
		max_x = std::max(max_x, x);
		min_y = std::min(min_y, y);
		max_y = std::max(max_y, y);
	}

	return std::tuple{ min_x, min_y, max_x, max_y };
}

void draw_line(int len) {
	std::cout << '+';
	for (int i = 0; i < len; ++i) {
		std::cout << '-';
	}
	std::cout << "+\n";
}

void canvas::draw() const {
	auto [min_x, min_y, max_x, max_y] = bounds();
	int len = max_x - min_x + 1;
	draw_line(len);
	for (int y = min_y; y <= max_y; ++y) {
		std::cout << '|';
		for (int x = min_x; x <= max_x; ++x) {
			std::cout << pixel(x, y);
		}
		std::cout << "|\n";
	}
	draw_line(len);
}

int main() {
	canvas c;
	c.pixel(1, 1) = 'X';
	c.pixel(2, 3) = 'Y';
	c.draw();

	const auto& rc = c;

	std::cout << '"'
	          << c.pixel(0, 0)
	          << rc.pixel(7, 7)
	          << std::as_const(c).pixel(5, 5)
	          << "\"\n";

	for (const auto& [xy, p] : c.pixels) {
		auto [x, y] = xy;
		std::cout << "(" << x << ", " << y << "): '" << p << "'\n";
	}

	c.clear();
	// rc.clear();  // error
}
