#include <iostream>
#include <map>
#include <vector>

struct counter {
	std::map<char, int> freq;

	void count(const std::vector<char>& vec) {
		for (char c : vec) {
			++freq[c];
		}
	}

	void print() const {
		std::cout << "{\n";
		for (auto& [key, value] : freq) {
			std::cout << "  '" << key << "': " << value << ",\n";
		}
		std::cout << "}\n";
	}

	counter& operator+=(const counter& other) {
		for (auto& [key, value] : other.freq) {
			freq[key] += value;
		}
		return *this;
	}

	friend counter operator+(counter lhs,
	                         const counter& rhs) {
		lhs += rhs;
		return lhs;
		// note: don't do the following, it makes an extra copy!
		//   return lhs += rhs;
	}
};


int main() {
	counter c1;
	c1.count({ 'H', 'e', 'l', 'l', 'o' });
	counter c2;
	c2.count({ 'w', 'o', 'r', 'l', 'd', '!' });
	counter c3;
	c3.count({ 'W', 'o', 'l', 'o', 'l', 'o', '!' });

	(c1 + c2 + c3).print();

	// note: (c1 + c2 + c3) is now basically the same as:
	//   counter tmp = c1;
	//   tmp += c2;
	//   tmp += c3;
}
